@@ -14,6 +14,7 @@
 //
 // $URL$
 // $Id$
+// SPDX-License-Identifier: LGPL-3.0+
 // 
 //
 // Author(s)     : Arno Eigenwillig <arno@mpi-inf.mpg.de>
@@ -30,6 +31,8 @@
 #ifndef CGAL_ALGEBRAIC_KERNEL_D_BITSTREAM_DESCARTES_RNDL_TREE_H
 #define CGAL_ALGEBRAIC_KERNEL_D_BITSTREAM_DESCARTES_RNDL_TREE_H
 
+#include <CGAL/disable_warnings.h>
+
 #include <vector>
 #include <list>
 #include <utility>
@@ -1506,7 +1509,7 @@ long Fujiwara_root_bound_log(
     RandomAccessIterator first, RandomAccessIterator beyond,
     LowerBoundLog2Abs lblog2, UpperBoundLog2AbsApproximator ublog2apx
 ) {
-    int n = beyond - first - 1; // degree
+    std::ptrdiff_t n = beyond - first - 1; // degree
     if (n < 1) return 0;
     long lblog2_lcoeff = lblog2(*(beyond - 1));
 
@@ -1516,7 +1519,7 @@ long Fujiwara_root_bound_log(
     std::vector<QE*> heap(n);    // heap is built from pointers to them
     for (int i = 0; i < n; ++i) {
         QE& entry = entries[i];
-        entry.n_minus_i = n - i;
+        entry.n_minus_i = static_cast<int>(n - i);
         entry.is_tight = ublog2apx.initial_upper_bound(
                 *(first + i), entry.ub_log2_qi, entry.is_certainly_zero
         );
@@ -1530,7 +1533,7 @@ long Fujiwara_root_bound_log(
     while (!heap[0]->is_tight) {
         std::pop_heap(heap.begin(), heap.end(), less);
         QE& popped = **(heap.end() - 1);
-        int i = n - popped.n_minus_i;
+        std::ptrdiff_t i = n - popped.n_minus_i;
         CGAL_assertion(i >= 0 && i < n);
         CGAL_assertion(&popped == &(entries[i]));
         popped.is_tight = ublog2apx.improve_upper_bound(
@@ -1568,6 +1571,8 @@ long Fujiwara_root_bound_log(
 
 } //namespace CGAL
 
+#include <CGAL/enable_warnings.h>
+
 #endif // CGAL_ALGEBRAIC_KERNEL_D_BITSTREAM_DESCARTES_RNDL_TREE_H
 
 // EOF
